unit CommPak;

interface

const

   RECEIVE    =  0;      { Receiver buffer register              }
   TRANSMIT   =  0;      { Transmitter holding register          }
   L_DIVISOR  =  0;      { Divisor latch, least significant byte }
   U_DIVISOR  =  1;      { Divisor latch, most significant byte  }
   INT_ENB    =  1;      { Interrupt enable register             }
   INT_ID     =  2;      { Interupt identification register      }
   LINE_CTRL  =  3;      { Line control register                 }
   MODEM_CTRL =  4;      { Modem control register                }
   LINE_STAT  =  5;      { Line status register                  }
   MODEM_STAT =  6;      { Modem status register                 }

   Even7      =  $1A;    { 11010 7 data bits, even parity        }
   None8      =  $03;    { 00011 8 data bits, no parity          }

procedure initialize(baud : word;parity:byte);
procedure RestoreInterrupt;
procedure SetInterrupt(irq : byte);
procedure ChangeInterrupt(new : byte);
function SetReceive : Byte;
function SetTransmit : Byte;
procedure WriteOUT;
procedure CheckForData;

implementation

uses Crt, Strings, Dos, ScrUnit, Globals, cppdelay;

var
   OldInterrupt : procedure;

{$F+}
procedure NewInterrupt; interrupt;
begin
   if ((port[BaseAddress+INT_ID] AND 4) = 4) then begin
      whatch := chr(port[BaseAddress + RECEIVE]);
      RString := RString + whatch;
      DataReady := TRUE;
   end;
{ sound(1000);delay(50);nosound;}
 if (IRQNum <= 7) then
    port[$20] := $20
 else begin
    port[$20] := $20;
    port[$A0] := $20;
 end;
end;
{$F-}

procedure CheckForData;
begin
   if ((port[BaseAddress+5{INT_ID}] AND 1{4}) = 1{4}) then begin
      whatch := chr(port[BaseAddress + RECEIVE]);
      RString := RString + whatch;
      DataReady := TRUE;
   end;
end;

procedure SetInterrupt(irq : byte);
var
   int_mask : word;
begin
   if (irq = 0) then
     Polling := TRUE
   else
     Polling := FALSE;

   if (Polling) then begin
     port[BaseAddress + INT_ENB] := 0;
     port[BaseAddress + MODEM_CTRL] := SetReceive;
     exit;
   end;
   if (irq <= 7) then begin
     GetIntVec(irq + 8, @oldinterrupt);
     asm cli; end;
     SetIntVec(irq + 8, Addr(newinterrupt));
     int_mask := port[$21];
     int_mask := int_mask AND NOT(1 SHL irq);
     port[$21] := int_mask;
     asm sti; end;
     port[$20] := $20;
     port[BaseAddress + INT_ENB] := 1;
     port[$20] := $20;
     port[BaseAddress + MODEM_CTRL] := SetReceive;
  end else begin
     GetIntVec(irq + $70 - 8, @oldinterrupt);
     asm cli; end;
     SetIntVec(irq + $70 - 8, Addr(newinterrupt));
     int_mask := port[$A1];
     int_mask := int_mask AND NOT(1 SHL (irq - 8));
     port[$A1] := int_mask;
     asm sti; end;
     port[$20] := $20;
     port[$A0] := $20;
     port[BaseAddress + INT_ENB] := 1;
     port[$20] := $20;
     port[$A0] := $20;
     port[BaseAddress + MODEM_CTRL] := SetReceive;
  end;

end;

procedure RestoreInterrupt;
var
   int_mask : word;
begin
   if (IRQNum = 0) then begin
     port[BaseAddress + INT_ENB] := 0;
     port[$20] := $20;
     port[BaseAddress + MODEM_CTRL] := SetReceive;
     exit;
   end;
   if (IRQNum <= 7) then begin
     port[BaseAddress + INT_ENB] := 0;
     port[$20] := $20;
     asm cli; end;
     int_mask := port[$21];
     int_mask := int_mask XOR (1 SHL IRQNum);
     port[$21] := int_mask;
     SetIntVec(IRQNum + 8, Addr(oldinterrupt));
     asm sti; end;
     port[$20] := $20;
     port[BaseAddress + MODEM_CTRL] := SetReceive;
   end else begin
     port[BaseAddress + INT_ENB] := 0;
     port[$20] := $20;
     port[$A0] := $20;
     asm cli; end;
     int_mask := port[$A1];
     int_mask := int_mask XOR (1 SHL (IRQNum - 8));
     port[$A1] := int_mask;
     SetIntVec(IRQNum + $70 - 8, Addr(oldinterrupt));
     asm sti; end;
     port[$20] := $20;
     port[$A0] := $20;
     port[BaseAddress + MODEM_CTRL] := SetReceive;
   end;
end;

procedure ChangeInterrupt(new : byte);
var
  myread : byte;
begin
   if (new = 0) then
     Polling := TRUE
   else
     Polling := FALSE;

   RestoreInterrupt;
   SetInterrupt(new);

   cdelay(50);
   myread := port[BaseAddress + RECEIVE];
   cdelay(50);
   myread := port[BaseAddress + RECEIVE];

   IRQNum := new;
end;

procedure initialize(baud : word;parity:byte);
var
  ch : INTEGER;
  divisor : word;
  X, Y : byte;
  dByte : byte;
  myread : byte;
begin
   baudrate := baud;
   if (parity<>$FF) then
      case parity of
              Even7 : begin
                         ParityMode := Even7;
                         ParityModeStr := '7E1';
                      end;
              None8 : begin
                         ParityMode := None8;
                         ParityModeStr := '8N1';
                      end;
           end;

   X := WhereX;
   Y := WhereY;
   updateScr;
   GotoXY(X, Y);
   if baud=0 then divisor:=1 else
     divisor := 115200 div baudrate;

   port[BaseAddress + LINE_CTRL] := $80;
   port[BaseAddress + L_DIVISOR] := lo(divisor);
   port[BaseAddress + U_DIVISOR] := hi(divisor);

   port[BaseAddress + LINE_CTRL] := ParityMode;

   port[BaseAddress + MODEM_CTRL] := SetReceive;

   cdelay(50);
   myread := port[BaseAddress + RECEIVE];
   cdelay(50);
   myread := port[BaseAddress + RECEIVE];

end;

Function SetReceive : Byte;
Begin
   SetReceive := 1 + Out1*4 + Out2*8;
End;

Function SetTransmit : Byte;
Begin
   SetTransmit := 1 + 2 + Out1*4 + Out2*8;
End;

Procedure WriteOUT;
Var
   myRead : byte;
Begin
   myRead := port[BaseAddress + MODEM_CTRL];
   myRead := (myRead AND $F3) OR (Out1*4 + Out2*8);
   port[BaseAddress + MODEM_CTRL] := myRead;
End;

begin

end.
