UNIT Mouse;  { Mouse interface module }

INTERFACE

USES Dos;

CONST
 LButton : Byte = 1;
 RButton : Byte = 2;
 BothButtons : Byte =3;

 PROCEDURE ResetMouse(var MousePresent : Boolean);
 PROCEDURE SetMouseSpeed(Horiz,Vert,DoubleThreshSpeed : Word);
 PROCEDURE SetActiveMousePage(Page : Word);
 FUNCTION GetActiveMousePage : Word;
 PROCEDURE GetMouseMotion(X,Y : Integer);
 FUNCTION MMouseX : Integer;
 FUNCTION MMouseY : Integer;
 PROCEDURE ShowCursor;
 PROCEDURE HideCursor;
 PROCEDURE ProtectWindow(X1,Y1,X2,Y2 : Word);
 PROCEDURE GetMouseStatus(X,Y : Word; Lb,Rb : Boolean);
 PROCEDURE BPushStatus(B : Word; Var Cur : Boolean; Var Np, lastX, lastY : Word );
 PROCEDURE BReleaseStatus(B : Word; Var Cur : Boolean; Var Nr, lastX, lastY : Word );
 PROCEDURE SetMouseWindow(X1,Y1,X2,Y2 : Word);
 PROCEDURE NewCursor(XHot,YHot : Word; var Bitmap );
 PROCEDURE LoadNewCursor(FilNam : String);
 PROCEDURE MoveMouseCursor(X,Y : Integer);
 FUNCTION LeftButton : Boolean;
 FUNCTION RightButton : Boolean;
 FUNCTION MouseX : Word;
 FUNCTION MouseY : Word;
IMPLEMENTATION

 PROCEDURE ResetMouse(var MousePresent : Boolean);
 VAR Regs : Registers;
 BEGIN
  With Regs do
   BEGIN
    Ax:=0;
    Intr($33,regs);
    If Ax=0 Then MousePresent:=False Else MousePresent:=True;
   END;
 END;

 PROCEDURE SetMouseSpeed(Horiz,Vert,DoubleThreshSpeed : Word);
 VAR Regs : Registers;
 BEGIN
  With Regs do
   BEGIN
    Ax:=26;
    Bx:=Horiz;
    Cx:=Vert;
    Dx:=DoubleThreshSpeed;
    Intr($33,Regs);
   END;
 END; { SetMouseSpeed }


 PROCEDURE ShowCursor;
 VAR Regs : Registers;
 BEGIN
  With Regs do
   BEGIN
    Ax:=1;
    Intr($33,Regs);
   END;
 END; {Show Cursor}

 PROCEDURE HideCursor;
 VAR Regs : Registers;
 BEGIN
  With Regs do
   BEGIN
    Ax:=2;
    Intr($33,Regs);
   END;
 END; { Hide Cursor }

 PROCEDURE ProtectWindow(X1,Y1,X2,Y2 : Word);
 VAR Regs : Registers;
 BEGIN
  With Regs do
   BEGIN
    Ax:=16;
    Cx:=X1;
    Dx:=Y1;
    Si:=X2;
    Di:=Y2;
    Intr($33,Regs);
   END;
 END; { ProtectWindow }

 PROCEDURE SetActiveMousePage(Page : Word);
 VAR Regs : Registers;
 BEGIN
  With Regs do
   BEGIN
    Ax:=29;
    Bx:=Page;
    Intr($33,Regs);
   END;
 END;

 FUNCTION GetActiveMousePage : Word;
 VAR Regs : Registers;
 BEGIN
  With Regs do
   BEGIN
    Ax:=30;
    Intr($33,Regs);
    GetActiveMousePage:=Bx;
   END;
 END;

 PROCEDURE GetMouseMotion(X,Y : Integer);
 VAR Regs : Registers;
 BEGIN
  With Regs do
   BEGIN
    Ax:=11;
    Intr($33,Regs);
    X:=Cx;
    Y:=Dx;
   END;
 END; { GetMouseMotion }

 FUNCTION MMouseX : Integer;
 VAR
  X,Y : Integer;
 BEGIN
  GetMouseMotion(X,Y);
  MMouseX:=X;
 END;

 FUNCTION MMouseY : Integer;
 VAR
  X,Y : Integer;
 BEGIN
  GetMouseMotion(X,Y);
  MMouseY:=Y;
 END;


 PROCEDURE GetMouseStatus(X,Y : Word; Lb,Rb : Boolean);
 VAR Regs : Registers;
 BEGIN
  With Regs do
   BEGIN
    Ax:=3;
    Intr($33,Regs);
    If (Bx=1) or (Bx=3) Then Lb:=True Else Lb:=False;
    If (Bx=2) or (Bx=3) Then Rb:=True Else Rb:=False;
    X:=Cx;
    Y:=Dx;
   END;
 END; { GetMouseStatus }

 PROCEDURE BPushStatus(B : Word; Var Cur : Boolean; Var Np, lastX, lastY : Word );
 VAR Regs : Registers;
 BEGIN
  With Regs do
   BEGIN
    Ax:=5;
    Bx:=B;
    Intr($33,Regs);
    If Ax=0 Then Cur:=False Else Cur:=True;
    Np:=Bx;
    lastX:=Cx;
    lastY:=Dx;
   END;
 END; { BPushStatus }

 PROCEDURE BReleaseStatus(B : Word; Var Cur : Boolean; Var Nr, lastX, lastY : Word );
 VAR Regs : Registers;
 BEGIN
  With Regs do
   BEGIN
    Ax:=6;
    Bx:=B;
    Intr($33,Regs);
    If Ax=0 Then Cur:=False Else Cur:=True;
    Nr:=Bx;
    lastX:=Cx;
    lastY:=Dx;
   END;
 END; { BReleaseStatus }

 PROCEDURE SetMouseWindow(X1,Y1,X2,Y2 : Word);
 VAR Regs : Registers;
 BEGIN
  With Regs do
   BEGIN
    Ax:=7;
    Cx:=X1;
    Dx:=X2;
    Intr($33,Regs);
    Ax:=8;
    Cx:=Y1;
    Dx:=Y2;
    Intr($33,Regs);
   END;
 END; { SetMouseWindow }

 PROCEDURE NewCursor(XHot,YHot : Word; var Bitmap );
 VAR
  Regs : Registers;
 BEGIN
  With Regs Do
   BEGIN
    Ax:=9;
    Bx:=XHot;
    Cx:=YHot;
    Dx:=Ofs(BitMap);
    Es:=Seg(BitMap);
    Intr($33,Regs);
   END;
 END; { NewCursor }

 PROCEDURE LoadNewCursor(FilNam : String);
 VAR
  F : File of Word;
  Cursor : Array [0..1,0..15] Of Word;
  H,V,W : Word;
  I,J : Integer;
 BEGIN
  Assign(F,FilNam);
  Reset(F);
  Read(F,W);
  H:=Hi(W);
  V:=Lo(W);
  For I:=0 to 1 do
   For J:=0 to 15 do
    Read(F,Cursor[I,J]);
  NewCursor(H,V,Cursor);
  Close(F);
 END;

 PROCEDURE MoveMouseCursor(X,Y : Integer);
 VAR
  R:Registers;
 BEGIN
  With R do
   BEGIN
    Ax:=4;
    Cx:=X;
    Dx:=Y;
    Intr($33,R);
   END;
 END;



 { Begin boolean function identifiers }



 FUNCTION LeftButton : Boolean;
 VAR Regs : Registers;
 BEGIN
  With Regs do
   BEGIN
    Ax:=3;
    Intr($33,Regs);
    If (Bx=1) or (Bx=3) Then LeftButton:=True else LeftButton:=False;
   END;
 END; { LeftButton }

 FUNCTION RightButton : Boolean;
 VAR Regs : Registers;
 BEGIN
  With Regs do
   BEGIN
    Ax:=3;
    Intr($33,Regs);
    If (Bx=2) or (Bx=3) Then RightButton:=True else RightButton:=False;
   END;
 END; { RightButton }

 FUNCTION MouseX : Word;
 VAR Regs : Registers;
 BEGIN
  With Regs do
   BEGIN
    Ax:=3;
    Intr($33,Regs);
    MouseX:=Cx;
   END;
 END; { MouseX }

 FUNCTION MouseY : Word;
 VAR Regs : Registers;
 BEGIN
  With Regs do
   BEGIN
    Ax:=3;
    Intr($33,Regs);
    MouseY:=Dx;
   END;
 END; { MouseY }

END.


