unit ScrUnit;

interface

   procedure SaveScr;
   procedure RestoreScr;
   procedure ScrollScr;
   procedure helpScr;
   procedure updateScr;
   procedure GetBaseAddress;
   procedure ExitProg;
   procedure JohnScroll;
   procedure EnterDebug;
   procedure ExitDebug;
   procedure WriteDebug(dc : char);

implementation

uses Crt, Strings, CommPak, Mouse, Globals;

var
   Screen : Array[0..7999] Of Byte;

procedure SaveScr;
var
  x : integer;
begin
  for x := 0 to (height*2*80)-1 do Screen[x] := Mem[$B800:x];
  OldX := WhereX;
  OldY := WhereY;
end;

procedure RestoreScr;
var
  x : integer;
begin
  for x := 0 to (height*2*80)-1 do Mem[$B800:x] := Screen[x];
  GotoXY(OldX, OldY);
end;

procedure ScrollScr;
var
  x,y : word;
begin
  y := $A0;
  x := 0;
  repeat
     MemL[$B800:x] := MemL[$B800:y];
     inc(y, 4);
     inc(x, 4);
  until x >= 3680;
end;

procedure JohnScroll; assembler;
asm
   push  ds
   push  es
   push  di
   push  si
   push  ax

   cld
   mov   ax,0B800h
   mov   ds,ax
   mov   es,ax
   mov   cx,1840
   mov   di,0
   mov   si,160
   rep   movsw

   pop   ax
   pop   si
   pop   di
   pop   es
   pop   ds
end;


procedure helpScr;
var
  thiskey : char;
begin
   Writeln('         ----------------> RISCTerm Help Screen <----------------');
   Writeln;
   Writeln('   To select the baud rate, press the function key corresponding');
   Writeln('   to the desired baud rate.');
   Writeln('                                               RISCTERM -i5 -a300 -b9600 -p7E1');
   Writeln('     F2 ------------>  1200 bits per second       Miscellaneous');
   Writeln('     F3 ------------>  2400 bits per second         Alt+B   Change Base Address');
   Writeln('     F4 ------------>  4800 bits per second         Alt+I   Change IRQ #');
   Writeln('     F5 ------------>  9600 bits per second         Alt+7,8 Select Parity');
   Writeln('     F6 ------------> 14400 bits per second         Alt+1,2 Change OUT1, OUT2');
   Writeln('     F7 ------------> 19200 bits per second                              ');
   Writeln('     F8 ------------> 28800 bits per second         Alt+D   Toggle Debug ');
   Writeln('     F9 ------------> 57600 bits per second                              ');
   Writeln;
   Writeln('Debug mode (Alt+D) allows you to see the hex value of the incoming characters.');
   Writeln;
   Writeln('OUT1 and OUT2 control how RISCTERM treats RTS, etc.                      ');
   Writeln;
   Writeln('   NOTE: Entering and leaving debug mode will clear the screen.          ');
   Writeln;
   Writeln('All options are also available with a mouse by clicking on the status line.');
   Writeln;
end;

procedure EnterDebug;
begin
  window(1, height-4, 80, height-1);
  TextColor(White);
  TextBackground(Magenta);
  clrscr;
  window(1, 1, 80, height-5);
  TextColor(White);
  TextBackground(Blue);
  clrscr;
  debugx := 1;
  debugy := 1;
end;

procedure ExitDebug;
begin
  window(1, 1, 80, height-1);
  TextColor(White);
  TextBackground(Blue);
  clrscr;
end;

procedure WriteDebug(dc : char);
var
  ox, oy : byte;
begin
  ox := WhereX; oy := WhereY;
  window(1, height-4, 80, height-1);
  GotoXY(debugx, debugy);
  TextColor(White);
  TextBackground(Magenta);
  write(HexAscii(byte(dc), 2), ' ');
  debugx := WhereX;
  debugy := WhereY;
  if (debugx = 2) then
    dec(debugx);
  window(1, 1, 80, height-5);
  TextColor(White);
  TextBackground(Blue);
  GotoXY(ox, oy);
end;

procedure updateScr;
var
  OX, OY : Byte;
  istr : string;
begin
   OX := WhereX;
   OY := WhereY;
   Window(1, height, 80, height);
   TextColor(Blue);
   TextBackground(White);
{   write('                                                                                ');
}   gotoxy(1,1);
   istr := HexAscii(IRQNum, 1);
   if (IRQNum = 0) then
     istr := 'X';

   write(' ');
   if (FoundMouse) then begin
     if (CurFunction = 1) then begin
       TextColor(Blue);
       TextBackground(Green);
     end;
     write(' ', HexAscii(BaseAddress,4), 'H ');
     TextColor(Blue);
     TextBackground(White);
     write('');
     if (CurFunction = 2) then begin
       TextColor(Blue);
       TextBackground(Green);
     end;
     write(' IRQ ', istr, ' ');
     TextColor(Blue);
     TextBackground(White);
     write('');
     if (CurFunction = 3) then begin
       TextColor(Blue);
       TextBackground(Green);
     end;
     write(' ', baudrate:5, ' Baud ');
     TextColor(Blue);
     TextBackground(White);
     write('');
     if (CurFunction = 4) then begin
       TextColor(Blue);
       TextBackground(Green);
     end;
     write(' ',ParityModeStr, ' ');
     TextColor(Blue);
     TextBackground(White);
     write('');
     if (CurFunction = 5) then begin
       TextColor(Blue);
       TextBackground(Green);
     end;
     write(Out1);
     TextColor(Blue);
     TextBackground(White);
     if (CurFunction = 6) then begin
       TextColor(Blue);
       TextBackground(Green);
     end;
     write(Out2);
     TextColor(Blue);
     TextBackground(White);
     write('');
     if (CurFunction = 7) then begin
       TextColor(Blue);
       TextBackground(Green);
     end;
     write(' F1 Help ');
     TextColor(Blue);
     TextBackground(White);
     write('');
     if (CurFunction = 8) then begin
       TextColor(Blue);
       TextBackground(Green);
     end;
     write(' Alt+C Clr Scr ');
     TextColor(Blue);
     TextBackground(White);
     write('');
     if (CurFunction = 9) then begin
       TextColor(Blue);
       TextBackground(Green);
     end;
     write(' Alt+X Exit ');
     TextColor(Blue);
     TextBackground(White);
   end else begin
     write(' ', HexAscii(BaseAddress,4), 'H  IRQ ', istr, '  ', baudrate:5, ' Baud ');
     write(' ',ParityModeStr, ' ', Out1,Out2,' F1 Help  Alt+C Clr Scr  Alt+X Exit ');
   end;
   write('');

   if (DebugMode) then
     window(1, 1, 80, height-5)
   else
     window(1, 1, 80, height-1);

   TextColor(White);
   TextBackground(Blue);
   GotoXY(OX, OY);

end;

procedure GetBaseAddress;
var
  key : char;
begin
  ClrScr;
  BaseAddress := AskForBaseAddress(BaseAddress);
  While (PortW[BaseAddress] = $FFFF) Do Begin
    WriteLn('Card was not found at that address!');
    WriteLn('Press a key to try again, or ESC to exit.');
    WriteLn;
    WriteLn('If you are certain that you received this message in error,');
    WriteLn('press SHIFT-O (capital letter "o") now to override and continue.');
    Key := ReadKey;
    if Key = 'O' then exit;
    If Key = #27 Then ExitProg;
    ClrScr;
    BaseAddress := AskForBaseAddress(BaseAddress);
  End;
end;

procedure ExitProg;
begin
   RestoreInterrupt;
   if FoundMouse then HideCursor;
   Window(1, 1, 80, height);
   TextColor(LightGray);
   TextBackground(Black);
   ClrScr;
   WriteLn('Exiting RISCTerm. . .');
   Halt(0);
end;

begin
end.